import numpy as np

from basics import *



#TODO: Check that the denominator value here is correct, is it biased is it not?
def semivariance(sample_data: np.ndarray, maximum_value = 0) -> float:
    use_data = sample_data[sample_data < maximum_value]
    n = use_data.size
    assert n > 0

    semivar = variance(use_data)

    return semivar



def simple_return() -> float:
    return 0



def compound_return() -> float:
    return 0


import numpy as np
import math
from pandas import NA
from scipy.stats import norm
from scipy.stats import t
from scipy.stats import chi2

from basics import *



def t_test(parameter_samples: np.ndarray, parameter_target=0, test_level=0.05, tail_number=2, tail_direction="upper", p_value_only=False):
    assert 0 <= test_level and test_level <= 1
    n = parameter_samples.size
    assert n > 1

    sample_mean = mean(parameter_samples)
    sample_variance = variance(parameter_samples)

    assert sample_variance >=0
    if sample_variance == 0:
        test_passed = False
        p_value = None
        #return "Insufficient Data"
        if p_value_only == True:
            return p_value
        elif p_value_only == False:
            return test_passed, p_value
        else:
            assert False

    test_statistic = (sample_mean - parameter_target) / (math.sqrt(sample_variance / n))

    if tail_number == 1:
        if tail_direction == "upper":
            critical_value = t.ppf(1-test_level, df=n-1)
            p_value = t.cdf(-test_statistic, df=n-1)

            if test_statistic <= critical_value:
                test_passed = False
            else:
                test_passed = True
        elif tail_direction == "lower":
            critical_value = t.ppf(test_level, df=n-1)
            p_value = t.cdf(test_statistic, df=n-1)

            if test_statistic >= critical_value:
                test_passed = False
            else:
                test_passed = True
        else:
            assert False
    elif tail_number == 2:
        critical_value_low  = t.ppf((test_level/2), df=n-1)
        critical_value_high = t.ppf(1-(test_level/2), df=n-1)
        p_value = 2 * t.cdf(-abs(test_statistic), df=n-1)

        if (test_statistic <= critical_value_high) and (test_statistic >= critical_value_low):
            test_passed = False
        else:
            test_passed = True
    else:
        assert False

    if p_value_only == True:
        return p_value
    elif p_value_only == False:
        return test_passed, p_value
    else:
        assert False



def z_test(parameter_samples: np.ndarray, parameter_target=0, test_level=0.05, tail_number=2, tail_direction="upper", p_value_only=False):
    assert 0 <= test_level and test_level <= 1
    n = parameter_samples.size
    assert n > 1

    sample_mean = mean(parameter_samples)
    sample_variance = variance(parameter_samples)
    
    assert sample_variance >= 0
    if sample_variance == 0:
        test_passed = False
        p_value = None
        #return "Insufficient Data"
        if p_value_only == True:
            return p_value
        elif p_value_only == False:
            return test_passed, p_value
        else:
            assert False

    test_statistic = (sample_mean - parameter_target) / math.sqrt(sample_variance)

    if tail_number == 1:
        if tail_direction == "upper":
            critical_value = norm.ppf(1 - test_level)
            p_value = norm.cdf(-test_statistic)

            if test_statistic <= critical_value:
                test_passed = False
            else:
                test_passed = True
        elif tail_direction == "lower":
            critical_value = norm.ppf(test_level)
            p_value = norm.cdf(test_statistic)

            if test_statistic >= critical_value:
                test_passed = False
            else:
                test_passed = True
        else:
            assert False
    elif tail_number == 2:
        critical_value_low  = norm.ppf((test_level/2))
        critical_value_high = norm.ppf(1-(test_level/2))
        p_value = 2 * norm.cdf(-abs(test_statistic))

        if (test_statistic <= critical_value_high) and (test_statistic >= critical_value_low):
            test_passed = False
        else:
            test_passed = True
    else:
        assert False

    if p_value_only == True:
        return p_value
    elif p_value_only == False:
        return test_passed, p_value
    else:
        assert False



def chi_sq_test():
    return 0



def f_test():
    return 0



def shapiro_wilk_test():
    return 0



def two_sample_t_test():
    return 0



#TODO: Check over this code
def confidence_interval(sample_data: np.ndarray, confidence_level=0.95, interval_type="mean", distribution = "normal"):
    assert 0 <= confidence_level and confidence_level <=1
    n = sample_data.size
    assert n > 1

    sample_mean = mean(sample_data)
    sample_variance = variance(sample_data)

    alpha = 1-confidence_level

    lower_level = alpha/2
    upper_level = 1 - alpha/2

    if distribution == "normal":
        lower_quantile = norm.ppf(lower_level)
        upper_quantile = norm.ppf(upper_level)
    elif distribution == "t":
        lower_quantile = t.ppf(lower_level,df=n-1)
        upper_quantile = t.ppf(upper_level,df=n-1)
    else:
        assert False

    if interval_type == "mean":
        lower_limit = sample_mean - upper_quantile*math.sqrt(sample_variance/n)
        upper_limit = sample_mean + upper_quantile*math.sqrt(sample_variance/n)
    elif interval_type == "variance":
        lower_quantile = chi2.ppf(upper_level,df=n-1)
        upper_quantile = chi2.ppf(lower_level,df=n-1)

        lower_limit = (n-1)*sample_variance/lower_quantile
        upper_limit = (n-1)*sample_variance/upper_quantile
    elif interval_type == "sd":
        #TODO: Double check that this is correct
        lower_quantile = chi2.ppf(upper_level,df=n-1)
        upper_quantile = chi2.ppf(lower_level,df=n-1)

        lower_limit = math.sqrt((n-1)*sample_variance/lower_quantile)
        upper_limit = math.sqrt((n-1)*sample_variance/upper_quantile)
    else:
        assert False

    return lower_limit, upper_limit



def anova():
    return 0



def manova():
    return 0



import numpy as np
import random as rd
import unittest

import sys

sys.path.insert(0, 'C:/Users/natji/statistical_methods/statistical_methods/')

from timeseries import *

class TestTimeseries(unittest.TestCase):

    def test_difference(self):
        assert True


if __name__ == '__main__':
    unittest.main()


import numpy as np
import random as rd
import unittest 

import sys

sys.path.insert(0, 'C:/Users/natji/statistical_methods/statistical_methods/')

from finance import *

class TestFinance(unittest.TestCase):

    def test_semivariance(self):
        assert True

if __name__ == '__main__':
    unittest.main()


from re import S
import numpy as np
import random as rd
import unittest

import sys

sys.path.insert(0, 'C:/Users/natji/statistical_methods/statistical_methods/')

from hypothesis_testing import *

class TestHypothesisTesting(unittest.TestCase):

    def test_t_test_zero_variance(self):
        data_number = rd.randint(2, 500)
        data_value  = rd.randrange(-500, 500)
        data = np.array([data_value] * data_number)

        p = t_test(data, p_value_only=True)

        self.assertIsNone(p)


    def test_t_test_further_from_target(self):
        data_number = rd.randint(2, 500)

        data1 = np.array([rd.randrange(-500, 500) for i in range(data_number)])

        target = mean(data1)

        data2 = data1 + 1
        data3 = data1 + 10
        data4 = data1 + 100

        p1 = t_test(data1, target, p_value_only=True)
        p2 = t_test(data2, target, p_value_only=True)
        p3 = t_test(data3, target, p_value_only=True)
        p4 = t_test(data4, target, p_value_only=True)
    
        assert p1 >= p2 >= p3 >= p4


    def test_t_test_mirrored_around_zero(self):
        data_number = rd.randint(2, 500)
        data1 = np.array([rd.randrange(0, 500) for i in range(data_number)])

        data2 = -data1

        p1 = t_test(data1, 0, tail_number=1, tail_direction="upper", p_value_only=True)
        p2 = t_test(data2, 0, tail_number=1, tail_direction="lower", p_value_only=True)

        self.assertEqual(p1, p2)


    def test_t_test_tail_number_changes(self):
        data_number = rd.randint(2, 500)
        data = np.array([rd.randrange(0, 500) for i in range(data_number)])

        p1 = t_test(data,0,tail_number=2,p_value_only=True)
        p2 = t_test(data,0,tail_number=1,tail_direction="upper",p_value_only=True)

        self.assertEqual(p1, p2*2)


    #TODO: Work out how to test for this
    def test_t_test_test_passess_correctly(self):
        pass_counter = 0
        fail_counter = 0
    
        for j in range(500):
            data_number = rd.randint(2,500)
            data = np.array([])
            for i in range(data_number):
                data = np.append(data,[rd.randrange(-500,500)])
        
            (test,p) = t_test(data)

            if p < 0.05:
                assert test == True
                pass_counter += 1
            elif p >= 0.05:
                assert test == False
                fail_counter += 1
            else:
                assert False

        assert True


    def test_t_test_p_value_sum(self):
        data_number = rd.randint(2, 500)
        data = np.array([rd.randrange(0, 500) for i in range(data_number)])

        p1 = t_test(data, 0, tail_number=1, tail_direction="upper", p_value_only=True)
        p2 = t_test(data, 0, tail_number=1, tail_direction="lower", p_value_only=True)

        self.assertEqual(p1 + p2, 1)


    def test_t_test_target_moves_closer(self):
        data_number = rd.randint(2, 500)
        data = np.array([rd.randrange(0, 500) for i in range(data_number)])
    
        target = mean(data)

        p1 = t_test(data, target, p_value_only=True)
        p2 = t_test(data, target+1, p_value_only=True)
        p3 = t_test(data, target+10, p_value_only=True)
        p4 = t_test(data, target+100, p_value_only=True)

        assert p1 >= p2 >= p3 >= p4


    #TODO: Work out how to test for this
    def test_ci_testing(self):
        passed = 0
        failed = 0

        for j in range(500):
            data_number = rd.randint(2, 500)
            data_1 = np.array([rd.randrange(0, 500) for i in range(data_number)])
            data_2 = np.array([rd.randrange(0, 500) for i in range(data_number)])

            (lower, upper) = confidence_interval(data_1)

            mean_2 = mean(data_2)

            if (mean_2 <= upper) and (mean_2>=lower):
                passed += 1
            else:
                failed += 1
        assert True
        

if __name__ == '__main__':
    unittest.main()


import numpy as np
import math 



def mean(sample_data: np.ndarray) -> float:
    n = sample_data.size
    assert n > 0

    sample_sum = 0
    for i in range(n):
        sample_sum += sample_data[i]

    sample_mean = sample_sum / n

    return sample_mean



def variance(sample_data: np.ndarray, use_biased_estimate=False) -> float:
    assert use_biased_estimate == True or use_biased_estimate == False
    n = sample_data.size
    assert n > 0

    if n == 1:
        use_biased_estimate = True

    sample_sum = 0
    sample_mean = mean(sample_data)

    for i in range(n):
        sample_sum += (sample_data[i] - sample_mean) ** 2

    if use_biased_estimate == True:
        sample_variance = sample_sum / n
    elif use_biased_estimate == False:
        sample_variance = sample_sum / (n-1)
    else:
        assert False
    
    return sample_variance



def standard_deviation(sample_data: np.ndarray, use_biased_estimate=False) -> float:
    assert use_biased_estimate == True or use_biased_estimate == False
    n = sample_data.size
    assert n > 0

    sample_variance = variance(sample_data, use_biased_estimate)
    sd = math.sqrt(sample_variance)

    return sd



def covariance(sample_x: np.ndarray, sample_y: np.ndarray, use_biased_estimate=False) -> float:
    assert use_biased_estimate == True or use_biased_estimate == False
    assert sample_x.size == sample_y.size
    n = sample_x.size
    assert n > 0 

    if n == 1:
        use_biased_estimate = True

    sample_sum = 0
    mean_x = mean(sample_x)
    mean_y = mean(sample_y)

    for i in range(n):
        sample_sum += sample_x[i] * sample_y[i]
    sample_sum = sample_sum - (mean_x * mean_y)

    if use_biased_estimate == True:
        sample_covariance = sample_sum / n
    elif use_biased_estimate == False:
        sample_covariance = sample_sum / (n-1)
    else:
        assert False
    
    return sample_covariance



def correlation(sample_x: np.ndarray, sample_y: np.ndarray, use_biased_estimate=False) -> float:
    assert use_biased_estimate == True or use_biased_estimate == False
    assert sample_x.size == sample_y.size
    n = sample_x.size
    assert n > 0 

    sd_x = standard_deviation(sample_x)
    sd_y = standard_deviation(sample_y)
    sample_cov = covariance(sample_x, sample_y, use_biased_estimate)

    sample_corr = sample_cov / (sd_x * sd_y)

    return sample_corr



#TODO: Needs testing
def skewness(sample_data: np.ndarray, use_biased_estimate=False) -> float:
    assert use_biased_estimate == True or use_biased_estimate == False
    n = sample_data.size
    assert n > 0 

    sample_mean = mean(sample_data)
    var = variance(sample_data, use_biased_estimate)

    third_moment_sum = 0

    for i in range(n):
        third_moment_sum += (sample_data[i] - sample_mean) ** 3

    third_moment = third_moment_sum / n

    skewness = third_moment / (var ** (3 / 2))

    return skewness



#TODO: change according to kurtosis defns
#TODO: what to do with biased estimates?
def kurtosis(sample_data: np.ndarray, use_excess_kurtosis=False) -> float:
    n = sample_data.size
    assert n > 0

    sample_mean = mean(sample_data)
    var = variance(sample_data, use_biased_estimate=True) 

    sample_sum = 0

    for i in range(n):
        sample_sum += (sample_data[i] - sample_mean) ** 4
    
    sample_sum = sample_sum / n

    if use_excess_kurtosis == True:
        sample_kurtosis = sample_sum / ((var) ** 2) - 3
    elif use_excess_kurtosis == False:
        sample_kurtosis = sample_sum / ((var) ** 2)
    else:
        assert False

    return sample_kurtosis



#TODO: Double check over moment code
#TODO: Centralised and standardized moments are not fully correct?
def moment(sample_data: np.ndarray, moment_power=1, use_standardized_estimate=False, use_centralized_estimate=False) -> float:
    n = sample_data.size
    assert n > 0
    assert moment_power >= 0 

    if (use_standardized_estimate == False) and (use_centralized_estimate == False):
        sample_sum = 0
        for i in range(n):
            sample_sum += (sample_data[i]) ** moment_power
        sample_moment = sample_sum / n
    elif use_standardized_estimate == True:
        sample_mean = mean(sample_data)
        sample_sd   = standard_deviation(sample_data)

        sample_sum = 0
        for i in range(n):
            sample_sum += ((sample_data[i] - sample_mean) / (sample_sd)) ** moment_power
        sample_moment = sample_sum / n
    elif (use_centralized_estimate == True) and (use_standardized_estimate == False):
        sample_mean = mean(sample_data)

        sample_sum = 0
        for i in range(n):
            sample_sum += (sample_data[i] - sample_mean) ** moment_power
        sample_moment = sample_sum / n
    else:
        assert False
    
    return sample_moment



def mse(modelled: np.ndarray, observed: np.ndarray) -> float:
    assert modelled.size == observed.size
    n = modelled.size
    assert n > 0 
    se_sum = 0

    for i in range(n):
        se_sum += (modelled[i] - observed[i]) ** 2

    mean_square_error = se_sum / n

    return mean_square_error



def rmse(modelled: np.ndarray, observed: np.ndarray) -> float:
    assert modelled.size == observed.size
    n = modelled.size
    assert n > 0

    mean_square_error = mse(modelled, observed)
    root_mean_square_error = math.sqrt(mean_square_error)

    return root_mean_square_error



def mean_error(modelled: np.ndarray, observed: np.ndarray) -> float:
    assert modelled.size == observed.size 
    n = modelled.size
    assert n > 0
    me_sum = 0 

    for i in range(n):
        me_sum += (modelled[i] - observed[i])
    
    me = me_sum/n

    return me



def data_range(sample_data: np.ndarray) -> float:
    upper = max(sample_data)
    lower = min(sample_data)
    sample_range = upper - lower
    return sample_range



#TODO: Look over, not the best method of calculating with small amounts of data
def percentile(sample_data: np.ndarray, percentage: float) -> float:
    assert 0 <= percentage and percentage <= 1
    sample_data.sort()
    n = sample_data.size

    if percentage == 0.5:
        return median(sample_data)
    elif percentage == 0:
        place = 0
    else:
        place = math.ceil(percentage * n) - 1

    percentile_out = sample_data[place]

    return percentile_out



def median(sample_data: np.ndarray) -> float:
    sample_data.sort()
    
    n = sample_data.size

    if (n % 2 == 0):
        upper_med = sample_data[n // 2]
        lower_med = sample_data[(n // 2) - 1]
        med = (upper_med + lower_med) / 2
    elif (n % 2 == 1):
        med = sample_data[n // 2]
    else:
        assert False

    return med



def iqr(sample_data: np.ndarray) -> float:
    upper_quartile = percentile(sample_data, 0.75)
    lower_quartile = percentile(sample_data, 0.25)

    i_q_r = upper_quartile - lower_quartile

    return i_q_r



def mode(sample_data: np.ndarray) -> np.ndarray:
    unique_data = set(sample_data)

    most_common_value = []
    mode_count = 0

    for value in unique_data:
        value_count = (sample_data == value).sum()

        if value_count > mode_count:
            most_common_value = [value]
            mode_count = value_count
        elif value_count == mode_count:
            most_common_value.append(value)

    return np.array(most_common_value)



import setuptools
import pathlib

here = pathlib.Path(__file__).parent.resolve()

with open("README.md", "r") as f:
    long_description = f.read()

long_description_ = (here / 'README.md').read_text()

setuptools.setup(
    name="statistical_methods",
    version="0.0.1",
    author="nbooth99",
    author_email="nathan_booth@outlook.com",
    description="Functions for Statistical Analysis",
    long_description_content_type="text/markdown",
    long_description=long_description_,
    python_requires=">=3.6",
    setup_requires =[],
    tests_require=[],
    install_requires=[],
    packages=setuptools.find_packages(),
    package_dir={'statistical_methods': 'statistical_methods'},
    classifiers=[
        "Programming Language :: Python :: 3.9",
        "Operating System :: OS Independent",
        "License :: OSI Approved :: MIT License"
    ],

)


import math
import numpy as np

from basics import *



def difference(series, order = 1, step = 1):
    n = series.size

    dif_number = math.floor((n-order)/step)
    assert dif_number >= 1

    if order == 1:
        dif = np.array([0]*math.floor((n-1)/step))
        
        for i in range(math.floor((n-1)/step)):
            dif[i] = series[i+step] - series[i]
    elif order > 1:
        temp_dif = np.array([0]*math.floor((n-1)/step))
        
        for i in range(math.floor((n-1)/step)):
            temp_dif[i] = series[i+step] - series[i]
        dif = difference(temp_dif, order = order-1, step = step)
    else:
        assert False

    return dif



import numpy as np
import random as rd
import unittest

#import sys

#sys.path.insert(0, 'C:/Users/natji/statistical_methods/statistical_methods/')

from statistical_methods.basics import *

class TestBasics(unittest.TestCase):

    def test_mean_two_repeated_values(self):
        data_number = rd.randint(1, 500)
        data = np.array([0, 1] * data_number)
        samp_mean = mean(data)
        self.assertEqual(samp_mean, 0.5)

    def test_mean_one_repeated_value(self):
        data_value = rd.randrange(-500, 500)
        data_number = rd.randint(1, 500)
        data = np.array([data_value] * data_number)
        samp_mean = mean(data)
        self.assertEqual(samp_mean, data_value)

    def test_mean_predetermined_data(self):
        data = np.array([5, 9, 7, 8.5, 4, 3, 2.1, 1.112, 5.8, 10, 0, 0.5, 5.5])
        samp_mean = mean(data)
        self.assertAlmostEqual(samp_mean, 4.73169230769)

    def test_mean_negative_data_negative_mean(self):
        data_number = rd.randint(1, 500)
        data = np.array([rd.randrange(-500, 500) for i in range(data_number)])
        data_neg = -data
        samp_mean = mean(data)
        mean_neg = mean(data_neg)
        self.assertEqual(samp_mean, -mean_neg)

    def test_mean_single_data_point(self):
        data_value = rd.randrange(-500, 500)
        data = np.array([data_value])
        samp_mean = mean(data)
        self.assertEqual(samp_mean, data_value)

    def test_variance_one_repeated_value(self):
        data_value = rd.randrange(-500, 500)
        data_number = rd.randint(1, 500)
        data = np.array([data_value] * data_number)
        var = variance(data)
        self.assertEqual(var, 0)

    def test_variance_repeating_two_values_more_decreases_variance(self):
        data_number = rd.randint(1, 500)
        data = np.array([rd.randrange(-500, 500) for i in range(data_number)])
        data_extended = data
        data_extended = np.append(data_extended,data)
        var = variance(data)
        var_extended = variance(data_extended)
        self.assertGreaterEqual(var, var_extended)

    def test_variance_single_data_point(self):
        data = np.array([rd.randrange(-500, 500)])
        var = variance(data)
        self.assertEqual(var, 0)

    def test_variance_predetermined_data(self):
        #TODO: Write predetermined data
        data = np.array([1, 1, 1])
        var = variance(data)
        self.assertEqual(var, 0)

    def test_variance_biased_estimate_differ(self):
        data_number = rd.randint(2,500)
        data = np.array([rd.randrange(-500, 500) for i in range(data_number)])

        var_1 = variance(data, use_biased_estimate=False)
        var_2 = variance(data, use_biased_estimate=True)

        self.assertGreaterEqual(var_1, var_2)

    def test_covariance(self):
        assert True

    def test_correlation(self):
        assert True

    def test_skewness(self):
        assert True

    def test_kurtosis(self):
        assert True

    def test_moment_first_moment_mean(self):
        data_number = rd.randint(2, 500)
        data = np.array([rd.randrange(-500, 500) for i in range(data_number)])
        samp_moment = moment(data)
        samp_mean = mean(data)
        self.assertEqual(samp_moment, samp_mean)

    def test_moment_first_central_moment_zero(self):
        data_number = rd.randint(2, 500)
        data = np.array([rd.randrange(-500, 500) for i in range(data_number)])
        samp_moment = moment(data, use_centralized_estimate= True)
        self.assertAlmostEqual(samp_moment, 0)

    def test_moment_second_centralised_moment_variance(self):
        data_number = rd.randint(2, 500)
        data = np.array([rd.randrange(-500, 500) for i in range(data_number)])
        samp_moment = moment(data, 2, use_centralized_estimate=True)
        var = variance(data, use_biased_estimate=True)
        self.assertEqual(samp_moment, var)

    def test_moment_zero_one(self):
        data_number = rd.randint(2, 500)
        data = np.array([rd.randrange(-500, 500) for i in range(data_number)])
        samp_moment = moment(data, 0)
        self.assertEqual(samp_moment, 1)

    def test_mse(self):
        assert True

    def test_rmse(self):
        assert True

    def test_mean_error(self):
        assert True

    def test_data_range(self):
        assert True

    def test_percentile(self):
        assert True

    def test_median(self):
        assert True

    def test_iqr(self):
        assert True

    def test_mode(self):
        assert True

    def test_correlation_correct_output_range(self):
        data_number = rd.randint(1, 500)

        data = np.array([rd.randrange(-500, 500) for i in range(data_number)])
        data2= np.array([rd.randrange(-500, 500) for j in range(data_number)])

        corr = correlation(data, data2)

        self.assertLessEqual(corr, 1)
        self.assertGreaterEqual(corr, -1)

    def test_correlation_identical_data(self):
        data_number = rd.randint(2, 500)

        data = np.array([rd.randrange(-500, 500) for i in range(data_number)])

        corr = correlation(data, data)

        self.assertAlmostEqual(corr, 1, places=1)

    def test_correlation_negative_data(self):
        data_number = rd.randint(2, 500)

        data = np.array([rd.randrange(-500, 500) for i in range(data_number)])

        corr = correlation(data, -data)

        self.assertAlmostEqual(corr, -1, places=1)

    def test_kurtosis_upper_bound(self):
        data_number = rd.randint(2, 500)

        data = np.array([rd.randrange(-500, 500) for i in range(data_number)])

        k = kurtosis(data)
        s = skewness(data)

        assert k >= ((s) ** 2) + 1

if __name__ == '__main__':
    unittest.main()